unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls;

const
  AM_ENDSTREAM = WM_USER + $99;
  AM_GOTDATA = WM_USER + $98;

type
  TMainForm = class(TForm)
    ControlPanel: TPanel;
    ReportMemo: TMemo;
    GoButton: TButton;
    StopButton: TButton;
    cbBipolar: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure GoButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
  protected
    procedure AMEndStream(var Msg: TMessage); message AM_ENDSTREAM;
    procedure AMGotData(var Msg: TMessage); message AM_GOTDATA;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  AIOUSB, DetectUnit;

const
  CallbacksPerGotData = 100;

type
  TUSBAI1616Config = packed record
    ChannelRange: array[$0..$F] of Byte;
    CalibMode: Byte;
    TrigMode: Byte;
    StartStopCh: Byte;
    Oversample: Byte;
    StartStopChEx: Byte; //added for compatibility with the AI-MUX family
  end;

var
  DeviceIndex: LongWord;
  Config: TUSBAI1616Config;
  NextChannel, NumChannels: Integer;
  LogFile: TFileStream;
  LogFN: AnsiString;
  LogScanBuffer: AnsiString;
  Callbacks, ScanNum: Integer;

procedure ADCallback(pBuf: PWord; BufSize, Flags, Context: LongWord); cdecl; forward;

function TimestampStr(const ST: TSystemTime): AnsiString;
begin
  //with ST do Result := Format('%.2d:%.2d:%.2d on %.2d of %.2d of %.4d', [ wHour, wMinute, wSecond, wDay, wMonth, wYear ]);
  with ST do Result := Format('%.2d:%.2d:%.2d', [ wHour, wMinute, wSecond ]);
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  DetectForm: TDetectForm;
  Status, PID, ConfigBufSize: LongWord;
  I: Integer;

begin
  IsMultiThread := True;

  DeviceIndex := diOnly;

  //If you only have the one board, QueryDeviceInfo() with diOnly will succeed
  //with the right PID. If you have more than one board, or haven't plugged it
  //in, it will fail, and this code will pop up the detector form so the user
  //can pick a board or cancel.
  Status := QueryDeviceInfo(DeviceIndex, @PID, nil, nil, nil, nil);
  if (Status <> ERROR_SUCCESS) or not (((PID >= $804F) and (PID <= $8053)) or ((PID >= $814F) and (PID <= $8153))) then begin
    DetectForm := TDetectForm.Create(Self);
    if DetectForm.ShowModal = mrOK then begin
      DeviceIndex := DetectForm.DeviceIndex;
      DetectForm.Free;
    end
    else begin
      DeviceIndex := diNone;
      DetectForm.Free;
      Application.Terminate;
      Exit;
    end;
  end;

  for I := $0 to $F do
    Config.ChannelRange[I] := $01
  ;
  Config.CalibMode := 0; //Take actual data, not internal calibration sources.
  Config.TrigMode := $15; //Scan selected channels each counter rising edge. Leave counter 0 external.
  Config.Oversample := 0; //No oversample.
  ConfigBufSize := SizeOf(Config);
  ADC_SetConfig(DeviceIndex, @Config, ConfigBufSize);
end;

procedure TMainForm.GoButtonClick(Sender: TObject);
var
  Status: LongWord;
  ConfigBufSize: LongWord;
  Hz: Double;
  ST: TSystemTime;
  I: Integer;
  StartChannel, EndChannel: Byte;

begin
  ReportMemo.Lines.Add('Acquiring...');

  //Stop the counter, in case it was running.
  Hz := 0;
  CTR_StartOutputFreq(DeviceIndex, 0, @Hz);


  //Set the A/D range to 0-10V, which will become 0-16V on this DPK.
  for I := $0 to $F do
    if cbBipolar.Checked then
      Config.ChannelRange[I] := $01
    else
      Config.ChannelRange[I] := $00
  ;
  NumChannels := 64;
  //Select all 64 channels.
  StartChannel := 0;
  EndChannel := NumChannels - 1;
  Config.StartStopCH := (EndChannel shl 4) or (StartChannel and $F);
  Config.StartStopChEx := (EndChannel and $F0) or (StartChannel shr 4);

  ConfigBufSize := SizeOf(Config);
  ADC_SetConfig(DeviceIndex, @Config, ConfigBufSize);

  ADC_SetCal(DeviceIndex, ':AUTO:'); //If the device doesn't support cal, ADC_SetCal just returns an error.

  NextChannel := 0;
  Callbacks := 0;

  GetLocalTime(ST);
  with ST do LogFN := IncludeTrailingBackslash(GetCurrentDir) + Format('%.4d%.2d%.2d_%.2d%.2d%.2d.csv', [ wYear, wMonth, wDay, wHour, wMinute, wSecond ]);
  LogFile := TFileStream.Create(LogFN, fmCreate);
  LogFile.Free;
  LogFile := TFileStream.Create(LogFN, fmOpenReadWrite or fmShareDenyWrite);
  LogScanBuffer := '"Scan#"';
  for I := StartChannel to EndChannel do
    LogScanBuffer := LogScanBuffer + (',"Ch ' + IntToStr(I) + '"')
  ;
  LogScanBuffer := LogScanBuffer + #13#10;
  LogFile.WriteBuffer(LogScanBuffer[1], Length(LogScanBuffer));
  ScanNum := 1;
  LogScanBuffer := IntToStr(ScanNum);

  Status := ADC_BulkContinuousCallbackStart(DeviceIndex, 1024, 64, 0, @ADCallback);
  if Status <> ERROR_SUCCESS then begin
    ReportMemo.Lines.Add('Error ' + IntToStr(Status) + ' starting acquisition.');
    LogFile.Free;
    Exit;
  end;

  //Since we've put it in scan mode, this is per channel, for 250kHz total.
  Hz := 250000/64;
  CTR_StartOutputFreq(DeviceIndex, 0, @Hz);

  ReportMemo.Lines.Add('Acquisition started at ' + TimestampStr(ST) + '.');

  GoButton.Enabled := False;
  StopButton.Enabled := True;
end;

procedure ADCallback(pBuf: PWord; BufSize, Flags, Context: LongWord); cdecl;
var
  iSample: Integer;

begin
  //Note: This is called from an alternate thread context.
  //(Specifically a worker thread spawned by the driver.)

  if BufSize <> 0 then begin
    for iSample := 0 to (BufSize div 2) - 1 do begin
      LogScanBuffer := LogScanBuffer + (',0x' + IntToHex(pBuf^, 4));
      Inc(pBuf);

      Inc(NextChannel);
      if NextChannel >= NumChannels then begin
        NextChannel := 0;

        LogScanBuffer := LogScanBuffer + #13#10;
        LogFile.WriteBuffer(LogScanBuffer[1], Length(LogScanBuffer));
        Inc(ScanNum);
        LogScanBuffer := IntToStr(ScanNum);
      end;
    end;
  end;

  if (Flags and 2) <> 0 then begin
    //This tells the main thread we've got the last continuous callback packet.
    PostMessage(MainForm.Handle, AM_ENDSTREAM, 0, 0);
  end;

  Inc(Callbacks);
  if Callbacks >= CallbacksPerGotData then begin
    Callbacks := 0;
    PostMessage(MainForm.Handle, AM_GOTDATA, 0, 0);
  end;
end;

procedure TMainForm.AMEndStream(var Msg: TMessage);
var
  ST: TSystemTime;

begin
  GetLocalTime(ST);
  ReportMemo.Lines.Add('Stream ended at ' + TimestampStr(ST) + '...');
end;

procedure TMainForm.StopButtonClick(Sender: TObject);
var
  EndStatus, IOStatus: LongWord;

begin
  ReportMemo.Lines.Add('Stopping...');
  EndStatus := ADC_BulkContinuousEnd(DeviceIndex, @IOStatus);

  Application.ProcessMessages; //Make sure we got the "stream ended" message first.

  if (EndStatus = ERROR_SUCCESS) and (IOStatus = ERROR_SUCCESS) then
    ReportMemo.Lines.Add('Acquisition complete.')
  else if EndStatus = ERROR_SUCCESS then
    ReportMemo.Lines.Add('Acquisition ended with I/O error ' + IntToStr(IOStatus) + '.')
  else
    ReportMemo.Lines.Add('Error ' + IntToStr(EndStatus) + ' ending acquisition.')
  ;

  FreeAndNil(LogFile);

  GoButton.Enabled := True;
  StopButton.Enabled := False;
end;

procedure TMainForm.AMGotData(var Msg: TMessage);
begin
end;

end.
